# Authors:
#   Jason Gerard DeRose <jderose@redhat.com>
#
# assets: Cache-friendly asset management via content-hash-naming
# Copyright (C) 2009  Red Hat
#
# This file is part of `assets`.
#
# `assets` is free software: you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# `assets` is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with `assets`.  If not, see <http://www.gnu.org/licenses/>.

"""
Test the `assetslib` package.
"""

import os
from os import path
try:
    from hashlib import sha1
except ImportError:
    from sha import new as sha1
from helpers import TempDir, raises
from assetslib import Assets


links = tuple('link_%d.tar.gz' % i for i in xrange(10))
files = tuple('file_%d.tar.gz' % i for i in xrange(10))
dirs = tuple('dir_%d.tar.gz' % i for i in xrange(10))
ghosts = tuple('ghost_%d.tar.gz' % i for i in xrange(10))  # ghosts don't exist


def create_dummy_layout(tmp):
    """
    Create a dummy layout of links, files, and dirs.
    """
    for i in xrange(10):
        l = links[i]
        f = files[i]
        d = dirs[i]
        os.symlink(f, tmp.join(l))
        tmp.touch(f)
        tmp.makedirs(d)
        assert path.islink(tmp.join(l))
        assert path.isfile(tmp.join(f)) and not path.islink(tmp.join(f))
        assert path.isdir(tmp.join(d)) and not path.islink(tmp.join(d))
        assert os.readlink(tmp.join(l)) == f


# Random base64-encoded data for asset
_data = """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"""
assets = tuple(
    _data + ('change hash %d' % i) for i in xrange(10)
)
hashes = tuple(
    sha1(data).hexdigest() for data in assets
)
assert len(set(hashes)) == len(hashes)
names = tuple(h + '.tar.gz' for h in hashes)


class test_Assets(object):
    """
    Test the `assetlib.Assets` class.
    """
    klass = Assets

    def test_init(self):
        """
        Test the `Assets.__init__` method.
        """
        inst = self.klass()
        d = TempDir()
        assert inst.url == '/_/'
        assert inst.dir.startswith('/tmp/assets.')
        assert path.isdir(inst.dir)
        assert inst.prod is True
        assert inst.hash is sha1

        # Test when dir is not a directory:
        not_dir = d.touch('not_dir')
        e = raises(ValueError, self.klass, dir=not_dir)
        assert str(e) == 'Assets.dir: not a directory: %r' % not_dir

        # Test providing all as kwargs:
        class MyHash(object):
            pass
        inst = self.klass(url='/assets', dir=d.path, prod=False, hash=MyHash)
        assert inst.url == '/assets/'
        assert inst.dir == d.path
        assert inst.prod is False
        assert inst.hash is MyHash

        # Test providing all as args:
        inst = self.klass('/srv/www///', d.path, False, MyHash)
        assert inst.url == '/srv/www/'
        assert inst.dir == d.path
        assert inst.prod is False
        assert inst.hash is MyHash

    def test_iter(self):
        """
        Test the `Assets.__iter__` method.
        """
        tmp = TempDir()
        inst = self.klass(dir=tmp.path)
        assert tuple(inst) == tuple()
        create_dummy_layout(tmp)
        assert tuple(inst) == links

    def test_contains(self):
        """
        Test the `Assets.__contains__` method.
        """
        tmp = TempDir()
        inst = self.klass(dir=tmp.path)
        for l in links:
            assert l not in inst
        create_dummy_layout(tmp)
        for l in links:
            assert l in inst
        for names in (files, dirs, ghosts):
            for n in names:
                assert n not in inst

    def test_getitem(self):
        """
        Test the `Assets.__getitem__` method.
        """
        # Test with prod=True
        tmp = TempDir()
        inst = self.klass(dir=tmp.path)
        for l in links:
            e = raises(KeyError, inst.__getitem__, l)
            assert str(e) == repr(l)
        create_dummy_layout(tmp)
        for i in xrange(10):
            l = links[i]
            f = files[i]
            assert inst[l] == f
        for names in (files, dirs, ghosts):
            for n in names:
                e = raises(KeyError, inst.__getitem__, n)
                assert str(e) == repr(n)

        # Test with prod=False
        tmp = TempDir()
        inst = self.klass(dir=tmp.path, prod=False)
        for l in links:
            e = raises(KeyError, inst.__getitem__, l)
            assert str(e) == repr(l)
        create_dummy_layout(tmp)
        for i in xrange(10):
            l = links[i]
            f = files[i]
            assert inst[l] == l
            assert inst[l] != f
        for names in (files, dirs, ghosts):
            for n in names:
                e = raises(KeyError, inst.__getitem__, n)
                assert str(e) == repr(n)

    def test_register(self):
        """
        Test the `Assets.register` method.
        """
        src = TempDir()
        for i in xrange(10):
            data = assets[i]
            l = links[i]
            d = dirs[i]
            src.write(data, l)  # Not a link here, but will be in Assets.dir
            src.makedirs(d)

        tmp = TempDir()
        inst = self.klass(dir=tmp.path)

        # Test with non-existent paths:
        for g in ghosts:
            p = src.join(g)
            e = raises(ValueError, inst.register, p)
            assert str(e) == 'filename: not a file: %r' % p

        # Test with directories:
        for d in dirs:
            p = src.join(d)
            e = raises(ValueError, inst.register, p)
            assert str(e) == 'filename: not a file: %r' % p
            assert path.isdir(p)

        # Test with a fileobj without read():
        e = raises(TypeError, inst.register, fileobj=True)
        assert str(e) == 'fileobj.read() not a callable'

        # Test with neither filename, fileobj, nor data:
        e = raises(TypeError, inst.register)
        assert str(e) == 'must provide either filename, fileobj, or data'

        # Test with fileobj but no key:
        for l in links:
            p = src.join(l)
            fp = open(p, 'r')
            e = raises(TypeError, inst.register, fileobj=fp)
            assert str(e) == 'must provide key if filename=None'
            fp.close()

        # Test with data but no key:
        e = raises(TypeError, inst.register, data='Some data')
        assert str(e) == 'must provide key if filename=None'

        # Test with filename:
        for l in links:
            p = src.join(l)
            assert inst.register(p) == l
        for i in xrange(10):
            l = links[i]
            n = names[i]
            data = assets[i]
            L = tmp.join(l)
            N = tmp.join(n)
            assert path.islink(L)
            assert os.readlink(L) == n
            assert path.isfile(N)
            assert not path.islink(N)
            assert open(N, 'rb').read() == data
        assert len(os.listdir(inst.dir)) == 20

        # Test with fileobj:
        tmp = TempDir()
        inst = self.klass(dir=tmp.path)
        for i in xrange(10):
            l = links[i]
            fp = open(src.join(l), 'rb')
            assert inst.register(fileobj=fp, key=l) == l
        for i in xrange(10):
            l = links[i]
            n = names[i]
            data = assets[i]
            L = tmp.join(l)
            N = tmp.join(n)
            assert path.islink(L)
            assert os.readlink(L) == n
            assert path.isfile(N)
            assert not path.islink(N)
            assert open(N, 'rb').read() == data
        assert len(os.listdir(inst.dir)) == 20

        # Test with data:
        tmp = TempDir()
        inst = self.klass(dir=tmp.path)
        for i in xrange(10):
            l = links[i]
            data = assets[i]
            assert inst.register(data=data, key=l) == l
        for i in xrange(10):
            l = links[i]
            n = names[i]
            data = assets[i]
            L = tmp.join(l)
            N = tmp.join(n)
            assert path.islink(L)
            assert os.readlink(L) == n
            assert path.isfile(N)
            assert not path.islink(N)
            assert open(N, 'rb').read() == data
        assert len(os.listdir(inst.dir)) == 20
